unit local_bus_ctlr;

interface

uses
  Classes, SysUtils, FileUtil,
//  CThreads,
  BaseUnix, sensor_types;

type

  // generic definition for local bus I/O
  TLocalBus_Ctlr = class(TComponent)
  public
     property fpHandle : Cint;
    property InitDone : boolean;
    property LastErrorNo : integer;
    property LastError : string;

    constructor Create(AOwner: TComponent; DevPath: string; ciDevAddr: CInt);
    destructor Destroy;
    function GetRegisterValue(DevReg: byte): byte; virtual; abstract;
    procedure SetRegisterValue(DevReg: byte; RegVal: byte); virtual; abstract;
    procedure GetRegistersValues(DevReg: byte; var RegsVals: array of byte); virtual; abstract;

  end;



  // I2C access
  TI2C_Ctl = class(TLocalBus_Ctlr)
  private
    FHandle: Cint;
    FInitDone: boolean;
    buf: array [0..1] of byte;
    FLastError: string;
    FLastErrorNo: integer;
  public
    property fpHandle : Cint read FHandle;
    property InitDone : boolean read FInitDone;
    property LastErrorNo : integer read FLastErrorNo;
    property LastError : string read FLastError;

    constructor Create(AOwner: TComponent; DevPath: string; ciDevAddr: CInt);
    destructor Destroy;
    function GetRegisterValue(DevReg: byte): byte; override;
    procedure SetRegisterValue(DevReg: byte; RegVal: byte); override;
    procedure GetRegistersValues(DevReg: byte; var RegsVals: array of byte); override;

  end;



  // SPI access
   SPI_Ctl = class(TLocalBus_Ctlr)
   private
     FHandle: Cint;
     FGPIOPin: string;
     FInitDone: boolean;
     buf: array [0..1] of byte;
     FLastError: string;
     FLastErrorNo: integer;
     procedure SetCS(OnOff: Boolean);
   public
     property fpHandle : Cint read FHandle;
     property InitDone : boolean read FInitDone;
     property LastErrorNo : integer read FLastErrorNo;
     property LastError : string read FLastError;

     constructor Create(AOwner: TComponent; DevPath: string; ciDevAddr: CInt);
     destructor Destroy;
     function GetRegisterValue(DevReg: byte): byte; override;
     procedure SetRegisterValue(DevReg: byte; RegVal: byte); override;
     procedure GetRegistersValues(DevReg: byte; var RegsVals: array of byte); override;

   end;


const
  RPi_I2C1_DevPath=      '/dev/i2c-1';



implementation

{ TLocalBus_Ctlr }
{==============================================================================}

constructor TLocalBus_Ctlr.Create(AOwner: TComponent; DevPath: string; ciDevAddr: CInt);
begin
  // init TComponent
  inherited Create(AOwner);
end;

destructor TLocalBus_Ctlr.Destroy;
begin
  // destroy TComponent
  inherited;
end;



{ TI2C_Ctl }
{==============================================================================}
 const
    I2C_SLAVE            = 1795;


constructor TI2C_Ctl.Create(AOwner: TComponent; DevPath: string; ciDevAddr: CInt);
begin
  inherited Create(AOwner, DevPath, ciDevAddr);
  try
    FHandle:= fpopen(DevPath, O_RDWR);
    fpIOCtl(FHandle, I2C_SLAVE, pointer(ciDevAddr));
    FInitDone:= True;
    FLastErrorNo:= -1;
    FLastError:= '';
  except
    FInitDone := False;
    //FLastErrorNo:= fpErrorNo;
    FLastError:= '';
  end;
end;

destructor TI2C_Ctl.Destroy;
begin
  if FInitDone then
  try
    fpclose(FHandle);
  finally
    inherited;
  end;
end;

function TI2C_Ctl.GetRegisterValue(DevReg: byte): byte;
var RegVal: byte;
begin
  try
    buf[0]:= DevReg;
    fpwrite(FHandle, buf[0], 1);
    fpread(FHandle, RegVal, 1);
  except
    //FLastErrorNo:= fpErrorNo;
    FLastError:= 'Error GetValue';
    Exit;
  end;
  Result:= RegVal;
end;

procedure TI2C_Ctl.SetRegisterValue(DevReg: byte; RegVal: byte);
begin
  try
    buf[0] := DevReg;
    buf[1] := RegVal;
    fpwrite(FHandle, buf[0], 2);
  except
    //FLastErrorNo:= fpErrorNo;
    FLastError:= 'Error SetValue';
  end;
end;


procedure TI2C_Ctl.GetRegistersValues(DevReg: byte; var RegsVals: array of byte);
begin
  try
    buf[0] := DevReg;
    fpwrite(FHandle, buf[0], 1);
    fpread(FHandle, RegsVals, Length(RegsVals));
  except
    //FLastErrorNo:= fpErrorNo;
    FLastError:= 'Error GetValue';
    Exit;
  end;
end;

{ SPI_Ctl }
{==============================================================================}
 const
    DefaultGPIOPath            = '/sys/class/gpio';


constructor SPI_Ctl.Create(AOwner: TComponent; DevPath: string; ciDevAddr: CInt);
var fileDesc : CInt;
begin
  inherited Create(AOwner, DevPath, ciDevAddr);
  try

    { Prepare SoC pin 18 (pin 12 on GPIO port) for access: }
    fileDesc := fpopen(DefaultGPIOPath + '/export', O_WrOnly);
//    fpwrite(fileDesc, PIN_18[0], 2);
    fpclose(fileDesc);
    { Set SoC pin 18 as input: }
    fileDesc := fpopen(DefaultGPIOPath + '/gpio18/direction', O_WrOnly);
//    fpwrite(fileDesc, IN_DIRECTION[0], 2);
    fpclose(fileDesc);



//    FHandle:= fpopen(DevPath, O_RDWR);
//    fpIOCtl(FHandle, I2C_SLAVE, pointer(ciDevAddr));
    FInitDone:= True;
    FLastErrorNo:= -1;
    FLastError:= '';
  except
    FInitDone := False;
    //FLastErrorNo:= fpErrorNo;
    FLastError:= '';
  end;
end;

destructor SPI_Ctl.Destroy;
var fileDesc : CInt;
begin
  if FInitDone then
  try
    fileDesc := fpopen(DefaultGPIOPath + '/unexport', O_WrOnly);
//    fpwrite(fileDesc, PIN_18[0], 2);
    fpclose(fileDesc);


    fpclose(FHandle);
  finally
    inherited;
  end;
end;

procedure SPI_Ctl.SetCS(OnOff: Boolean);
var
  fileDesc: CInt;
  sValue: string[1];
begin
  if OnOff
    then sValue[1]:= '1'
    else sValue[1]:= '0';
  try
    { Open SoC pin 18 (pin 12 on GPIO port) in read-only mode: }
    fileDesc := fpopen(DefaultGPIOPath + '/gpio18/value', O_RdOnly);
    if fileDesc > 0 then
    begin
      { Read status of this pin (0: button pressed, 1: button released): }
      fpwrite(fileDesc, sValue[1], 1);
    end;
  finally
    { Close SoC pin 18 (pin 12 on GPIO port) }
    fpclose(fileDesc);
  end;
end;

function SPI_Ctl.GetRegisterValue(DevReg: byte): byte;
var RegVal: byte;
begin
  try
    buf[0]:= DevReg;
    fpwrite(FHandle, buf[0], 1);
    fpread(FHandle, RegVal, 1);
  except
    //FLastErrorNo:= fpErrorNo;
    FLastError:= 'Error GetValue';
    Exit;
  end;
  Result:= RegVal;
end;

procedure SPI_Ctl.SetRegisterValue(DevReg: byte; RegVal: byte);
begin
  try
    buf[0] := DevReg;
    buf[1] := RegVal;
    fpwrite(FHandle, buf[0], 2);
  except
    //FLastErrorNo:= fpErrorNo;
    FLastError:= 'Error SetValue';
  end;
end;


procedure SPI_Ctl.GetRegistersValues(DevReg: byte; var RegsVals: array of byte);
begin
  try
    buf[0] := DevReg;
    fpwrite(FHandle, buf[0], 1);
    fpread(FHandle, RegsVals, Length(RegsVals));
  except
    //FLastErrorNo:= fpErrorNo;
    FLastError:= 'Error GetValue';
    Exit;
  end;
end;


end.
