unit sensor_lps22hb;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil,
  BaseUnix, local_bus_ctlr, sensor_types;

type
  //  ---------------  Hardware définition  ------------------------------------
  // LPS22HB registers
  TLPS22HB_Regs= (INTERRUPT_CFG,
                  THS_P_L,
                  THS_P_H,
                  WHO_AM_I,
                  CTRL_REG1,
                  CTRL_REG2,
                  CTRL_REG3,
                  FIFO_CTRL,
                  REF_P_XL,
                  REF_P_L,
                  REF_P_H,
                  RPDS_P_L,
                  RPDS_P_H,
                  RES_CONF,
                  INT_SOURCE,
                  FIFO_STATUS,
                  STATUS,
                  PRESS_OUT_XL,
                  PRESS_OUT_L,
                  PRESS_OUT_H,
                  TEMP_OUT_L,
                  TEMP_OUT_H,
                  LPFP_RES
                 );

const
  // I2C address
  LPS22HB_Addr0: byte=    $5C;                 // if SA0= 0
  LPS22HB_Addr1: byte=    $5D;                 // if SA0= 1

  // Registers addresses
  RegsAddr: array[TLPS22HB_Regs] of byte=
                 ($0b,      // INTERRUPT_CFG
                  $0c,      // THS_P_L
                  $0d,      // THS_P_H
                  $0f,      // WHO_AM_I
                  $10,      // CTRL_REG1
                  $11,      // CTRL_REG2
                  $12,      // CTRL_REG3
                  $14,      // FIFO_CTRL
                  $15,      // REF_P_XL
                  $16,      // REF_P_L
                  $17,      // REF_P_H
                  $18,      // RPDS_P_L
                  $19,      // RPDS_P_H
                  $1a,      // RES_CONF
                  $25,      // INT_SOURCE
                  $26,      // FIFO_STATUS
                  $27,      // STATUS
                  $28,      // PRESS_OUT_XL
                  $29,      // PRESS_OUT_L
                  $2a,      // PRESS_OUT_H
                  $2b,      // TEMP_OUT_L
                  $2c,      // TEMP_OUT_H
                  $33       // LPFP_RES
                 );



      // Registers datas
      //CTRL_REG1_G
      gRate_14_9Hz: byte= 32;//b5;
      gScale_245: byte= 0;
      gBandWidth0: byte= 0;
  //    CRTL_REG1_G_Default: byte= gRate_14_9Hz;  // or gScale_245 or gBandWidth0
      CTRL_REG1_G_Default: byte= 32;  // ie or gScale_245 or gBandWidth0


//  ---------------  Hardware définition  ------------------------------------

 type
  TLPS22HB = class(TComponent)
  private
    { Déclarations privées }
    FP_IO_Ctlr: TLocalBus_Ctlr;
    FScale: single;
    function GetRegister(ARegister: TLPS22HB_Regs): byte;
    procedure SetRegister(ARegister: TLPS22HB_Regs; AValue: byte);
    function GetTemp: single;
    function GetPressure: single;

  protected
    { Déclarations protégées }
  public
    { Déclarations publiques }
    // I/O controllers
    property Pressure_IO_Ctlr: TLocalBus_Ctlr read FP_IO_Ctlr write FP_IO_Ctlr;
    // direct regiters access (no control)
    property Register[r: TLPS22HB_Regs]: byte read GetRegister write SetRegister;
    // high level access to datas
//    property FullScale: single read FFullScale;
    property Temperature: single read GetTemp;
    property Pressure: single read GetPressure;


    constructor Create(AOwner: TComponent);
    destructor Destroy;

    procedure SetDefaultOp;

  published
    { Déclarations publiées }
  end;



implementation


{ TLPS22HB }
{==============================================================================}

constructor TLPS22HB.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TLPS22HB.Destroy;
begin
  inherited;
end;


procedure TLPS22HB.SetDefaultOp;
begin
  try
    //
//    SetRegister(CTRL_REG1, CTRL_REG1_Default);

    FScale:= 1 / 40.96;   // 1 hPa fo 4096 LSB
  except
//    FInitDone := False;
  end;
end;

procedure TLPS22HB.SetRegister(ARegister: TLPS22HB_Regs; AValue: byte);
begin
  FP_IO_Ctlr.SetRegisterValue(RegsAddr[ARegister], AValue);
end;

function TLPS22HB.GetRegister(ARegister: TLPS22HB_Regs): byte;
begin
  FP_IO_Ctlr.GetRegisterValue(RegsAddr[ARegister]);
end;

function TLPS22HB.GetTemp: single;
var iTemp: Int16_4IO;
begin
  FP_IO_Ctlr.GetRegistersValues(RegsAddr[TEMP_OUT_L], iTemp.bytes);
  Result:= 25. + single(iTemp.intValue) / 16.;
end;

function TLPS22HB.GetPressure: single;
// returns the pressure in Pa
var iP: Int32_4IO;
    bP: array[0..2] of byte;
begin
  FP_IO_Ctlr.GetRegistersValues(RegsAddr[PRESS_OUT_XL], bP);
  ip.bytes[0]:= bP[2];
  ip.bytes[1]:= bP[1];
  ip.bytes[2]:= bP[0];
  ip.bytes[3]:= 0;
  Result:= FScale * single(iP.intValue);
end;



end.
