unit sensor_lsm9ds1;

interface

uses
  Classes, SysUtils, FileUtil,
  BaseUnix, local_bus_ctlr, sensor_types;

type
//- Hardware définition ---------------------------------------------- begin -//
  // LSM9DS1 registers
  TLSM9DS1_Regs= (ACT_THS,
                  ACT_DUR,
                  INT_GEN_CFG_XL,
                  INT_GEN_THS_X_XL,
                  INT_GEN_THS_Y_XL,
                  INT_GEN_THS_Z_XL,
                  INT_GEN_DUR_XL,
                  REFERENCE_G,
                  INT1_CTRL,
                  INT2_CTRL,
                  WHO_AM_I,
                  CTRL_REG1_G,
                  CTRL_REG2_G,
                  CTRL_REG3_G,
                  ORIENT_CFG_G,
                  INT_GEN_SRC_G,
                  OUT_TEMP_L,
                  OUT_TEMP_H,
                  STATUS_REG17,
                  OUT_X_G_L,
                  OUT_X_G_H,
                  OUT_Y_G_L,
                  OUT_Y_G_H,
                  OUT_Z_G_L,
                  OUT_Z_G_H,
                  CTRL_REG4,
                  CTRL_REG5_XL,
                  CTRL_REG6_XL,
                  CTRL_REG7_XL,
                  CTRL_REG8,
                  CTRL_REG9,
                  CTRL_REG10,
                  INT_GEN_SRC_XL,
                  STATUS_REG27,
                  OUT_X_XL_L,
                  OUT_X_XL_H,
                  OUT_Y_XL_L,
                  OUT_Y_XL_H,
                  OUT_Z_XL_L,
                  OUT_Z_XL_H,
                  FIFO_CTRL,
                  FIFO_SRC,
                  INT_GEN_CFG_G,
                  INT_GEN_THS_X_G_L,
                  INT_GEN_THS_X_G_H,
                  INT_GEN_THS_Y_G_L,
                  INT_GEN_THS_Y_G_H,
                  INT_GEN_THS_Z_G_L,
                  INT_GEN_THS_Z_G_H,
                  INT_GEN_DUR_G,
                  // Magnétique (M)
                  OFFSET_X_REG_L_M,
                  OFFSET_X_REG_H_M,
                  OFFSET_Y_REG_L_M,
                  OFFSET_Y_REG_H_M,
                  OFFSET_Z_REG_L_M,
                  OFFSET_Z_REG_H_M,
                  WHO_AM_I_M,
                  CTRL_REG1_M,
                  CTRL_REG2_M,
                  CTRL_REG3_M,
                  CTRL_REG4_M,
                  CTRL_REG5_M,
                  STATUS_REG_M,
                  OUT_X_L_M,
                  OUT_X_H_M,
                  OUT_Y_L_M,
                  OUT_Y_H_M,
                  OUT_Z_L_M,
                  OUT_Z_H_M,
                  INT_CFG_M,
                  INT_SRC_M,
                  INT_THS_L_M,
                  INT_THS_H_M
                 );

  // LSM9DS1 scale foctors
  TLSM9DS1_Scales= record
    XL: single;
    G: single;
    M: single;
    T: single;
  end;

const
  // I2C address
  LSM9DS1_XL_G_Addr0: byte= $6A;                 // if SDO_A/G= 0
  LSM9DS1_M_Addr0: byte=    $1C;                 // if SDO_M= 0
  LSM9DS1_XL_G_Addr1: byte= $6B;                 // if SDO_A/G= 1
  LSM9DS1_M_Addr1: byte=    $1E;                 // if SDO_M= 1

  // Registers addresses
  RegsAddr: array[TLSM9DS1_Regs] of byte=
                 (// Accéléro (XL) and gyro (G)
                  $04,      // ACT_THS
                  $05,      // ACT_DUR
                  $06,      // INT_GEN_CFG_XL
                  $07,      // INT_GEN_THS_X_XL
                  $08,      // INT_GEN_THS_Y_XL
                  $09,      // INT_GEN_THS_Z_XL
                  $0a,      // INT_GEN_DUR_XL
                  $0b,      // REFERENCE_G
                  $0c,      // INT1_CTRL
                  $0d,      // INT2_CTRL
                  $0f,      // WHO_AM_I
                  $10,      // CTRL_REG1_G
                  $11,      // CTRL_REG2_G
                  $12,      // CTRL_REG3_G
                  $13,      // ORIENT_CFG_G
                  $14,      // INT_GEN_SRC_G
                  $15,      // OUT_TEMP_L
                  $16,      // OUT_TEMP_H
                  $17,      // STATUS_REG17
                  $18,      // OUT_X_G_L
                  $19,      // OUT_X_G_H
                  $1a,      // OUT_Y_G_L
                  $1b,      // OUT_Y_G_H
                  $1c,      // OUT_Z_G_L
                  $1d,      // OUT_Z_G_H
                  $1e,      // CTRL_REG4
                  $1f,      // CTRL_REG5_XL
                  $20,      // CTRL_REG6_XL
                  $21,      // CTRL_REG7_XL
                  $22,      // CTRL_REG8
                  $23,      // CTRL_REG9
                  $24,      // CTRL_REG10
                  $26,      // INT_GEN_SRC_XL
                  $27,      // STATUS_REG27
                  $28,      // OUT_X_XL_L
                  $29,      // OUT_X_XL_H
                  $2a,      // OUT_Y_XL_L
                  $2b,      // OUT_Y_XL_H
                  $2c,      // OUT_Z_XL_L
                  $2d,      // OUT_Z_XL_H
                  $2e,      // FIFO_CTRL
                  $2f,      // FIFO_SRC
                  $30,      // INT_GEN_CFG_G
                  $31,      // INT_GEN_THS_X_G_L
                  $32,      // INT_GEN_THS_X_G_H
                  $33,      // INT_GEN_THS_Y_G_L
                  $34,      // INT_GEN_THS_Y_G_H
                  $35,      // INT_GEN_THS_Z_G_L
                  $36,      // INT_GEN_THS_Z_G_H
                  $37,      // INT_GEN_DUR_G
                  // Magnétique (M)
                  $05,      // OFFSET_X_REG_L_M
                  $06,      // OFFSET_X_REG_H_M
                  $07,      // OFFSET_Y_REG_L_M
                  $08,      // OFFSET_Y_REG_H_M
                  $09,      // OFFSET_Z_REG_L_M
                  $0a,      // OFFSET_Z_REG_H_M
                  $0f,      // WHO_AM_I_M
                  $20,      // CTRL_REG1_M
                  $21,      // CTRL_REG2_M
                  $22,      // CTRL_REG3_M
                  $23,      // CTRL_REG4_M
                  $24,      // CTRL_REG5_M
                  $27,      // STATUS_REG_M
                  $28,      // OUT_X_L_M
                  $29,      // OUT_X_H_M
                  $2a,      // OUT_Y_L_M
                  $2b,      // OUT_Y_H_M
                  $2c,      // OUT_Z_L_M
                  $2d,      // OUT_Z_H_M
                  $30,      // INT_CFG_M
                  $31,      // INT_SRC_M
                  $32,      // INT_THS_L_M
                  $33       // INT_THS_H_M
                 );



      // Registers settings (partial implementation)
      //CTRL_REG1_G : gyro rate / scale / bandwith
      gRate_0Hz: byte=                 0; // default : power down
      gRate_14_9Hz: byte=      %00100000;
      gRate_59_5Hz: byte=      %01000000;
      gRate_119Hz: byte=       %01100000;
      gRate_238Hz: byte=       %10000000;
      gRate_476Hz: byte=       %10100000;
      gRate_952Hz: byte=       %11000000;
      gScale_245: byte=                0; // default
      gScale_500: byte=        %00001000;
      gScale_2000: byte=       %00011000;
      gBandWidth0: byte=               0; // default

      //CTRL_REG6_XL : accelero rate / scale /bandwith
      xlRate_0Hz: byte=                0; // default : power down
      xlRate_10Hz: byte=       %00100000;
      xlRate_50Hz: byte=       %01000000;
      xlRate_119Hz: byte=      %01100000;
      xlRate_238Hz: byte=      %10000000;
      xlRate_476Hz: byte=      %10100000;
      xlRate_952Hz: byte=      %11000000;
      xlScale_2g: byte=                0; // default
      xlScale_4g: byte=        %00010000;
      xlScale_8g: byte=        %00011000;
      xlScale_16g: byte=       %00001000;
      xlBandWidthODR: byte=            0; // default bandwith by ODR
      xlAntiAliasBW408Hz: byte=        0; // default

       //CTRL_REG7_XL
       xlHighResolution: byte=  %10000000; // (disabled by default)
       xlCutoffFreq: byte=              0; // default
       xlFilteredOutput: byte=          0; // default
       xlFilteredInterrupt: byte=       0; // default

      //CTRL_REG8
       sBlockUpdate: byte=      %01000000; // datas not updated until MSB and LSB read
       sAddrInc: byte=          %00000100; // increment address on multiple read
       sBigLittleEndian: byte=  %00000010; // big/little endian 1= MSB @ lower address

       //STATUS_REG17 or STATUS_REG27
       xlIntGenerated: byte=    %01000000; // accelero interrupt generated
       gIntGenerated: byte=     %00100000; // gyro interrupt generated
       sIntInactivity: byte=    %00010000; // inactivity interrupt generated
       sBootStatus: byte=       %00001000; // if boot running
       tDataAvailable: byte=    %00000100; // temperature data available
       gDataAvailable: byte=    %00000010; // gyro data available
       xlDataAvailable: byte=   %00000001; // accelero data available


       //CTRL_REG1_M : magneto rate, performance, compensation
       mTempComp: byte=         %10000000; // temperature compensation
       mXYLowPower: byte=               0; // default
       mXYMediumPerf: byte=     %00100000; // medium performance on X and Y axes
       mXYHighPerf: byte=       %01000000; // high performance on X and Y axes
       mXYUltraPerf: byte=      %01100000; // ultra-high performance on X and Y axes
       mRate_0_625Hz: byte=             0; // 0.625 Hz output rate, default
       mRate_1_25Hz: byte=      %00000100; // 1.25 Hz output rate
       mRate_2_5Hz: byte=       %00001000; // 2.5 Hz output rate
       mRate_5Hz: byte=         %00001100; // 5 Hz output rate
       mRate_10Hz: byte=        %00010000; // 10 Hz output rate
       mRate_20Hz: byte=        %00010100; // 20 Hz output rate
       mRate_40Hz: byte=        %00011000; // 40 Hz output rate
       mRate_80Hz: byte=        %00011100; // 80 Hz output rate
       mSelTest: byte=          %00000001; // self test

       //CTRL_REG2_M : magneto scale
       mScale4G: byte=                  0; // default : 4 gauss
       mScale8G: byte=          %00100000; // 8 gauss
       mScale12G: byte=         %01000000; // 12 gauss
       mScale16G: byte=         %01100000; // 16 gauss

       //CTRL_REG3_M : operation mode
       mModeContinu: byte=              0; // continuous conversion mode
       mModeSingle: byte=       %00000001; // single conversion mode
       mModePowerDown: byte=    %00000010; // power down
       mModePowerDownD: byte=   %00000011; // defaut : power down

       //CTRL_REG4_M : operation mode
       mZLowPower: byte=                0; // default
       mZMediumPerf: byte=      %00000100; // medium performance on X and Y axes
       mZHighPerf: byte=        %00001000; // high performance on X and Y axes
       mZUltraPerf: byte=       %00001100; // ultra-high performance on X and Y axes
       mBigLittleEndian: byte=  %00000010; // big/little endian 1= MSB @ lower address

       //CTRL_REG5_M : operation mode
       mBlockUpdate: byte=      %01000000; // datas not updated until MSB and LSB read

       //STATUS_REG_M
       mxyzDataOverun: byte=    %10000000; // new magneto xyz data has overwritten previous datas
       mzDataOverun: byte=      %01000000; // new magneto z data has overwritten previous datas
       myDataOverun: byte=      %01000000; // new magneto y data has overwritten previous datas
       mxDataOverun: byte=      %01000000; // new magneto x data has overwritten previous datas
       mxyzDataAvailable: byte= %00001000; // magneto xyz data available
       mzDataAvailable: byte=   %00000100; // magneto z data available
       myDataAvailable: byte=   %00000010; // magneto y data available
       mxDataAvailable: byte=   %00000001; // magneto x data available

//- Hardware définition ------------------------------------------------ end -//

type
  // LSM9DS1 high level access
  TLSM9DS1 = class(TComponent)
  private
    FXL_G_IO: TLocalBus_Ctlr;
    FM_IO: TLocalBus_Ctlr;
    FFullScales: TLSM9DS1_Scales;
    FScales: TLSM9DS1_Scales;
    function GetRegister(ARegister: TLSM9DS1_Regs): byte;
    procedure SetRegister(ARegister: TLSM9DS1_Regs; AValue: byte);
    function GetTemp: single;
    function GetXL: V3D_Single;
    function GetG: V3D_Single;
    function GetM: V3D_Single;

   public
     // I/O controllers
    property XL_G_IO_Ctlr: TLocalBus_Ctlr read FXL_G_IO write FXL_G_IO;
    property M_IO_Ctlr: TLocalBus_Ctlr read FM_IO write FM_IO;
    // direct regiters access (no control)
    property Register[r: TLSM9DS1_Regs]: byte read GetRegister write SetRegister;
    // high level access to datas
    property FullScales: TLSM9DS1_Scales read FFullScales;
    property Temperature: single read GetTemp;
    property Acceleration: V3D_Single read GetXL;
    property Gyro: V3D_Single read GetG;
    property Magneto: V3D_Single read GetM;


    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function CheckWhoAmI(var waiXL_G: byte; var waiM: byte): boolean;
    function DatasReady(var dXL_Ready: boolean; var dG_Ready: boolean; var dM_Ready: boolean): boolean;
    procedure SetDefaultOp;

  end;



implementation


{ TLSM9DS1 }
{==============================================================================}

constructor TLSM9DS1.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TLSM9DS1.Destroy;
begin
  inherited;
end;


procedure TLSM9DS1.SetDefaultOp;
// Init the LSM9DS1 to operate
const g: single= 9.80665;
var CTRL_REG1_G_Default: byte;
    CTRL_REG6_XL_Default: byte;
    CTRL_REG7_XL_Default: byte;
    CTRL_REG8_Default: byte;
    CTRL_REG1_M_Default: byte;
    CTRL_REG3_M_Default: byte;
    CTRL_REG4_M_Default: byte;
    CTRL_REG5_M_Default: byte;
begin
  // set the regsiters and scales variables
  try
    // CTRL_REG1_G : gyro output rate/ scale / bandwith
    CTRL_REG1_G_Default := gRate_952Hz; // ie or gScale_245 or gBandWidth0
    SetRegister(CTRL_REG1_G, CTRL_REG1_G_Default);
    FFullScales.G:= 4.2760567;    // 245 °/s = 4.27605666739 rad/s
    FScales.G:= FFullScales.G / High(SmallInt);
    // CTRL_REG6_XL : accelero output rate/ scale / bandwith
    CTRL_REG6_XL_Default:= xlRate_952Hz or xlScale_4g;
    SetRegister(CTRL_REG6_XL, CTRL_REG6_XL_Default);
    FFullScales.XL:= 4.*g;
    FScales.XL:= FFullScales.XL / High(SmallInt);
    // CTRL_REG7_XL : high resolution for acceleration
    CTRL_REG7_XL_Default:= xlHighResolution;
    SetRegister(CTRL_REG7_XL, CTRL_REG7_XL_Default);
    // CTRL_REG8 : bloc update and incremental access
    CTRL_REG8_Default:= sBlockUpdate or sAddrInc;
    SetRegister(CTRL_REG8, CTRL_REG8_Default);


    // CTRL_REG1_M : magneto compensation / perf / rate
    CTRL_REG1_M_Default:= mTempComp or mXYUltraPerf or mRate_80Hz;
    SetRegister(CTRL_REG1_M, CTRL_REG1_M_Default);

    // CTRL_REG2_M : magneto scale
    // use default scale : nothing to set
    FFullScales.M:= 0.0004;  // 4 gauss = 0.0004 tesla
    FScales.M:= FFullScales.M / High(SmallInt);

     // CTRL_REG3_M : operation mode rate
    CTRL_REG3_M_Default:= mModeContinu;
    SetRegister(CTRL_REG3_M, CTRL_REG3_M_Default);

    // CTRL_REG4_M : Z axis performance
    CTRL_REG4_M_Default:= mZUltraPerf;
    SetRegister(CTRL_REG4_M, CTRL_REG4_M_Default);

    // CTRL_REG5_M : magneto bloc update
    CTRL_REG5_M_Default:= mBlockUpdate;
    SetRegister(CTRL_REG5_M, CTRL_REG5_M_Default);


   // temperature
    FScales.T:= 1 / 16.;   // 16 LSB /°

  except
//    FInitDone := False;
  end;
end;

function TLSM9DS1.GetRegister(ARegister: TLSM9DS1_Regs): byte;
begin
  if ARegister < OFFSET_X_REG_L_M
    then Result:= XL_G_IO_Ctlr.GetRegisterValue(RegsAddr[ARegister])
    else M_IO_Ctlr.GetRegisterValue(RegsAddr[ARegister]);
end;

procedure TLSM9DS1.SetRegister(ARegister: TLSM9DS1_Regs; AValue: byte);
begin
  if ARegister < OFFSET_X_REG_L_M
    then XL_G_IO_Ctlr.SetRegisterValue(RegsAddr[ARegister], AValue)
    else M_IO_Ctlr.SetRegisterValue(RegsAddr[ARegister], AValue);
end;

function TLSM9DS1.CheckWhoAmI(var waiXL_G: byte; var waiM: byte): boolean;
// returns true if we read correct value on both interfaces
// waiXL_G returns WHO_AM_I register, waiM returns WHO_AM_I_M register
const
  waiXL_G_Value: byte= $68;    // correct value for WHO_AM_I
  waiM_Value: byte=    $3d;    // correct value for WHO_AM_I_M
var wai: array[0..1] of byte;
begin
  wai[0]:= XL_G_IO_Ctlr.GetRegisterValue(RegsAddr[WHO_AM_I]);
  wai[1]:= M_IO_Ctlr.GetRegisterValue(RegsAddr[WHO_AM_I_M]);
  waiXL_G:= wai[0];
  waiM:= wai[1];
  Result:= (wai[0] = waiXL_G_Value) and (wai[1] = waiM_Value);
end;

function TLSM9DS1.DatasReady(var dXL_Ready: boolean; var dG_Ready: boolean; var dM_Ready: boolean): boolean;
// returns true if new XL, G and M datas are available
// dXL_Ready true if new accelero (XL) datas are available
// dG_Ready true if new gyro (G) datas are available
// dM_Ready true if new magneto (M) datas are available
var daReg: array[0..1] of byte;
begin
  daReg[0]:= XL_G_IO_Ctlr.GetRegisterValue(RegsAddr[STATUS_REG17]);
  daReg[1]:= M_IO_Ctlr.GetRegisterValue(RegsAddr[STATUS_REG_M]);
  // check data available bits
  dXL_Ready:= (daReg[0] and xlDataAvailable) = xlDataAvailable;
  dG_Ready:= (daReg[0] and gDataAvailable) = gDataAvailable;
  dM_Ready:= (daReg[1] and mxyzDataAvailable) = mxyzDataAvailable;
  Result:= dXL_Ready and dG_Ready and dM_Ready;
end;

function TLSM9DS1.GetTemp: single;
// returns the temperature in °K
const Temp0: single= 273.15 + 25.;
var iTemp: Int16_4IO;
begin
  XL_G_IO_Ctlr.GetRegistersValues(RegsAddr[OUT_TEMP_L], iTemp.bytes);
  Result:= Temp0 + FScales.T * single(iTemp.intValue);
end;

function TLSM9DS1.GetXL: V3D_Single;
// returns the acceleration vector in m / s2
var iXL: V3D_Int16_4IO;
begin
  XL_G_IO_Ctlr.GetRegistersValues(RegsAddr[OUT_X_XL_L], iXL.bytes);
  Result.X:= FScales.XL * single(iXL.intValues.X);
  Result.Y:= FScales.XL * single(iXL.intValues.Y);
  Result.Z:= FScales.XL * single(iXL.intValues.Z);
end;

function TLSM9DS1.GetG: V3D_Single;
// returns the gyro vector in rad / s
var iG: V3D_Int16_4IO;
begin
  XL_G_IO_Ctlr.GetRegistersValues(RegsAddr[OUT_X_G_L], iG.bytes);
  Result.X:= FScales.G * single(iG.intValues.X);
  Result.Y:= FScales.G * single(iG.intValues.Y);
  Result.Z:= FScales.G * single(iG.intValues.Z);
end;

function TLSM9DS1.GetM: V3D_Single;
// returns the magnetic vector in T
var iM: V3D_Int16_4IO;
begin
  M_IO_Ctlr.GetRegistersValues(RegsAddr[OUT_X_L_M], iM.bytes);
  Result.X:= FScales.M * single(iM.intValues.X);
  Result.Y:= FScales.M * single(iM.intValues.Y);
  Result.Z:= FScales.M * single(iM.intValues.Z);
end;

end.
