unit sqlite_logger;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, sqlite3conn, sqldb;

type
  //
  TDataTableDescriptor= array of string;
  //
  TSQLiteLogger = class(TComponent)
  private
    slDB: TSQLite3Connection;
    slDefaultTransaction: TSQLTransaction;
    slQuery: TSQLQuery;
    procedure SetDB(DBFileName: string);
    function GetDB: string;
    procedure SetActive(OnOff: boolean);
    function GetActive: boolean;
  public
    property DB: string read GetDB write SetDB;
    property Active: boolean read GetActive write SetActive;

    constructor Create(AOwner: TComponent; ADB: string);
    destructor Destroy; override;

    procedure CreateDataTable(TableDescriptor: TDataTableDescriptor);
    procedure RecordDatas(ATime: TDateTime; Datas: array of single);
  end;


implementation

constructor TSQLiteLogger.Create(AOwner: TComponent; ADB: string);
begin
  // init TComponent
  inherited Create(AOwner);
  // create components
  slDB:= TSQLite3Connection.Create(Self);
  slDB.DatabaseName:= ADB;
  slDefaultTransaction:= TSQLTransaction.Create(Self);
  slDB.Transaction:= slDefaultTransaction;
  slQuery:= TSQLQuery.Create(Self);
  slQuery.Database:= slDB;
end;

destructor TSQLiteLogger.Destroy;
var i: integer;
begin
  slDefaultTransaction.Commit;
  for i:= ComponentCount-1 downto 0 do Components[i].Free;
  // destroy TComponent
  inherited;
end;



procedure TSQLiteLogger.SetDB(DBFileName: string);
begin
  if DBFileName <> slDB.DatabaseName
  then begin
    slDB.Close;
    slDB.DatabaseName:= DBFileName;
  end;
end;

function TSQLiteLogger.GetDB: string;
begin
  Result:= slDB.DatabaseName;
end;

procedure TSQLiteLogger.SetActive(OnOff: boolean);
begin
  if OnOff
    then slDB.Open
    else slDB.Close;
end;

function TSQLiteLogger.GetActive: boolean;
begin
//  Result:= slDB.Open;
end;


procedure TSQLiteLogger.CreateDataTable(TableDescriptor: TDataTableDescriptor);
// create the data table : first column is time stamp + n data columns
var i: integer;
    sCreate, sIns: string;
begin
  // the create command
  sCreate:= 'create table Measures('+
                    'M_Time DateTime NOT NULL, ';
  for i:= Low(TableDescriptor) to High(TableDescriptor) do
  begin
    sCreate:= sCreate + TableDescriptor[i] + ' float NOT NULL';
    if i <> High(TableDescriptor)
      then sCreate:= sCreate + ','
      else sCreate:= sCreate + ');' ;
  end;

  try
    slDefaultTransaction.Active := True;

    // create the table
    slDB.ExecuteDirect(sCreate);

    // indexing upon time stamp
    slDB.ExecuteDirect('create unique index "ixM_Time" on "Measures"("M_Time");');

    slDefaultTransaction.Commit;
  except
    Exception.Create('Unable to create table');
  end;

  // the insert command
  sIns:= 'insert into Measures values(:t, ';
  for i:= Low(TableDescriptor) to High(TableDescriptor) do
  begin
    sIns:= sIns + ':m' + IntToStr(i);
    if i <> High(TableDescriptor)
      then sIns:= sIns + ', '
      else sIns:= sIns + ');' ;
  end;
  slQuery.SQL.Add(sIns);
end;


procedure TSQLiteLogger.RecordDatas(ATime: TDateTime; Datas: array of single);
var i: integer;
begin
  slQuery.Params[i].asDateTime:= ATime;
  for i:= Low(Datas) to High(Datas)
    do slQuery.Params[i+1].asFloat:= Datas[i];
  slQuery.ExecSQL;
end;

end.

