unit ti_main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  Buttons, ExtCtrls, baseUnix, sensor_lsm9ds1, local_bus_ctlr, sensor_types,
  sqlite_logger;

type

  { TfTest }
  TfTest = class(TForm)
    btnAccelero: TButton;
    btnGyro: TButton;
    btnLog: TButton;
    btnMagneto: TButton;
    btnQuitButton1: TButton;
    btnTemperature: TButton;
    btnWhoAmI: TButton;
    btnReg: TButton;
    eGX: TEdit;
    eGY: TEdit;
    eGZ: TEdit;
    eMModule: TEdit;
    eMX: TEdit;
    eMY: TEdit;
    eMZ: TEdit;
    eTemp: TEdit;
    eWhoAmIOK: TEdit;
    eReg: TEdit;
    eXLModule: TEdit;
    eXLX: TEdit;
    eWhoAmI: TEdit;
    eXLY: TEdit;
    eWhoAmIM: TEdit;
    eXLZ: TEdit;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    lEnregistrement: TLabel;
    sbnContinu: TSpeedButton;
    procedure btnAcceleroClick(Sender: TObject);
    procedure btnGyroClick(Sender: TObject);
    procedure btnLogClick(Sender: TObject);
    procedure btnMagnetoClick(Sender: TObject);
    procedure btnQuitButton1Click(Sender: TObject);
    procedure btnTemperatureClick(Sender: TObject);
    procedure btnRegClick(Sender: TObject);
    procedure btnWhoAmIClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure sbnContinuClick(Sender: TObject);
  private
    { private declarations }
    XLG_IO_Ctlr: TI2C_Ctl;
    M_IO_Ctlr: TI2C_Ctl;
    Logger: TSQLiteLogger;
    Reg4Test: TLSM9DS1_Regs;
    sReg4Test: string;
    tmContinu: TTimer;
    procedure tmContinuTimer(Sender: TObject);
  public
    { public declarations }
  end;

var
  fTest: TfTest;
  AGSensor: TLSM9DS1;

implementation

{$R *.lfm}

{ TfTest }

procedure TfTest.FormCreate(Sender: TObject);
begin
  ThousandSeparator:= ' ';
  DecimalSeparator:= ',';
  // initialisation du capteur
  AGSensor:= TLSM9DS1.Create(Self);
  // assign HW IO controler (the sensor is on I2C-1)
  XLG_IO_Ctlr:= TI2C_Ctl.Create(Self, RPi_I2C1_DevPath, LSM9DS1_XL_G_Addr1);
  AGSensor.XL_G_IO_Ctlr:= XLG_IO_Ctlr;
  M_IO_Ctlr:= TI2C_Ctl.Create(Self, RPi_I2C1_DevPath, LSM9DS1_M_Addr1);
  AGSensor.M_IO_Ctlr:= M_IO_Ctlr;
  // notre configuration par défaut
  AGSensor.SetDefaultOp;
  // choix du registre pour le test (modifier les 2 lignes ci-desous)
  Reg4Test:= CTRL_REG7_XL;
  sReg4Test:= 'CTRL_REG7_XL';
  btnReg.Caption:= Format('Registre %s (0x%x)', [sReg4Test, RegsAddr[Reg4Test]]);
end;

procedure TfTest.btnQuitButton1Click(Sender: TObject);
begin
  XLG_IO_Ctlr.Free;
  M_IO_Ctlr.Free;
  AGSensor.Free;
  Close;
end;


procedure TfTest.btnWhoAmIClick(Sender: TObject);
var waiXG, waiM : byte;
    isOK: boolean;
begin
  // vérifie que
  isOK:= AGSensor.CheckWhoAmI(waiXG, waiM);
  if isOK
    then eWhoAmIOK.Text:= 'OK'
    else eWhoAmIOK.Text:= 'not OK!';
  eWhoAmI.Text:= ToS0bxx(waiXG);
  eWhoAmIM.Text:= ToS0bxx(waiM);
end;

procedure TfTest.btnRegClick(Sender: TObject);
var val : byte;
begin
  val := AGSensor.Register[Reg4Test];
  eReg.Text:= ToS0bxx(val);
end;


procedure TfTest.btnTemperatureClick(Sender: TObject);
var temp: single;
begin
  temp:= AGSensor.Temperature;
  eTemp.Text:= Format('%f [°C]', [temp-273.15]);
end;

procedure TfTest.btnAcceleroClick(Sender: TObject);
var vXL: V3D_Single;
    module: single;
begin
  vXL:= AGSensor.Acceleration;
  module := sqrt( sqr(vXL.X) + sqr(vXL.Y) + sqr(vXL.Z) );
  eXLX.Text:= Format('%f [m/s²]', [vXL.X]);
  eXLY.Text:= Format('%f [m/s²]', [vXL.Y]);
  eXLZ.Text:= Format('%f [m/s²]', [vXL.Z]);
  eXLModule.Text:= Format('Module: %f [m/s²]', [module]);
end;

procedure TfTest.btnGyroClick(Sender: TObject);
const rad2deg: single = 180./Pi;   // 57.29578
var vG: V3D_Single;
begin
  vG:= AGSensor.Gyro;
  eGX.Text:= Format('%f [°/s]', [vG.X*rad2deg]);
  eGY.Text:= Format('%f [°/s]', [vG.Y*rad2deg]);
  eGZ.Text:= Format('%f [°/s]', [vG.Z*rad2deg]);
end;

procedure TfTest.btnMagnetoClick(Sender: TObject);
var vM: V3D_Single;
    module: single;
begin
  vM:= AGSensor.Magneto;
  module := sqrt( sqr(vM.X) + sqr(vM.Y) + sqr(vM.Z) );
  eMX.Text:= Format('%5.0n [nT]', [vM.X*1e9]);
  eMY.Text:= Format('%5.0n [nT]', [vM.Y*1e9]);
  eMZ.Text:= Format('%5.0n [nT]', [vM.Z*1e9]);
  eMModule.Text:= Format('Module: %5.0n [nT]', [module*1e9]);
end;

procedure TfTest.sbnContinuClick(Sender: TObject);
begin
  if tmContinu = nil then
  begin
    tmContinu:= TTimer.Create(Self);
    tmContinu.Interval:= 500;       // 0.5 s
    tmContinu.Ontimer:= @tmContinuTimer;
  end;
  tmContinu.Enabled:= not tmContinu.Enabled;
  sbnContinu.Down:= tmContinu.Enabled;
end;

procedure TfTest.tmContinuTimer(Sender: TObject);
begin
  // simule le fonctionnement des boutons
  btnTemperatureClick(Sender);
  btnAcceleroClick(Sender);
  btnGyroClick(Sender);
  btnMagnetoClick(Sender);
end;


procedure TfTest.btnLogClick(Sender: TObject);
type
  TDataRecord = record
    case ArrayACCess: boolean of
      True: (DataArray: array[0..9] of single);
      False: (Sequence: single; vXL: V3D_Single; vG: V3D_Single; vM: V3D_Single);
    end;
const
  TableColumns: array[0..9] of string=
     ('M_SeqNo', 'M_XL_X', 'M_XL_Y', 'M_XL_Z',
                 'M_G_X', 'M_G_Y', 'M_G_Z',
                 'M_M_X', 'M_M_Y', 'M_M_Z');
  sEnregistrement= 'Enregistrement en cours: %d';
var s, i, nbSeq: integer;
    TheDatas: TDataRecord;
    drxl, drg, drm: boolean;
begin
  Logger:= TSQLiteLogger.Create(Self, '/home/pi/MLogs/LSM9DS1Calibration.s3db');
  Logger.CreateDataTable(TableColumns);
  nbSeq:= 6;
  for s:= 1 to nbSeq do
  begin
    i:= 1;
    TheDatas.Sequence:= s;
    while True do
    begin
      if AGSensor.DatasReady(drxl, drg, drm)
      then begin
        lEnregistrement.Caption:= Format(sEnregistrement, [i]);
        Refresh;
        TheDatas.vXL:= AGSensor.Acceleration;
        TheDatas.vG:= AGSensor.Gyro;
        TheDatas.vM:= AGSensor.Magneto;
        Logger.RecordDatas(now, TheDatas.DataArray);
        i:= i+1;
        if i > 1000 then break;
      end;
    end;
    if s < NbSeq
      then ShowMessage(Format('Mettez la capteur dans une nouvelle position (séquence %d/%d)', [s+1, nbSeq]));
  end;
  Logger.Free;
  lEnregistrement.Caption:= 'Enregistrement effectué';
end;


end.

